fn main() {
    let input = std::fs::read_to_string("input.txt").unwrap();

    let res = part_num_sum(&input);

    println!("Result: {}", res);
}

fn part_num_sum(input: &str) -> u32 {
    let grid: Vec<&str> = input.split_whitespace().collect();

    dbg!(&grid);

    let mut numbers = vec![];
    for (y, line) in grid.iter().enumerate() {
        let mut start = None;
        let mut end = 0;
        for (x, c) in line.chars().enumerate() {
            if !c.is_numeric() {
                if let Some(s) = start {
                    start = None;
                    numbers.push(((s as i32, y as i32), &line[s..=end]));
                }
                continue;
            }
            if start.is_some() {
                end = x;
            }
            else {
                start = Some(x);
                end = x;
            }
        }
        if let Some(s) = start {
            start = None;
            numbers.push(((s as i32, y as i32), &line[s..]));
        }
    }

    let width = grid[0].len();
    let height = grid.len();

    let mut parts = vec![];

    for ((x, y), num) in numbers {
        let len = num.len() as i32;
        let left = x - 1;
        let right = x + len;
        let top = y - 1;
        let bottom = y + 1;

        let positions = (left..=right).map(|x| (x, top))
            .chain([(left, y), (right, y)])
            .chain((left..=right).map(|x| (x, bottom)));

        for (x, y) in positions {
            println!("Checking {}: ({},{})", num, x,y);
            if x < 0 || x >= width as i32
            || y < 0 || y >= height as i32 {
                continue;
            }
            let c = grid[y as usize].chars().nth(x as usize).unwrap();
            if !c.is_ascii_digit() && !c.is_whitespace() && c != '.' {
                println!("Part found: ({},{}), {}", x,y, num);
                let part: u32 = num.parse().unwrap();
                parts.push(part);
                break;
            }
        }
    }


    dbg!(parts).iter().sum()
}

#[cfg(test)]
mod test {
    use crate::part_num_sum;


    const EXAMPLE: &str =
"467..114..
...*......
..35..633.
......#...
617*......
.....+.58.
..592.....
......755.
...$.*....
.664.598..";

    #[test]
    fn example_1() {
        assert_eq!(4361, part_num_sum(EXAMPLE));
    }
}
